### 01 Masterscript
### 20.07.2021

###################################################################################################
### ANALOGUE METHOD
###################################################################################################
# Calibration of the analog method (data.frame)
specifications <- list(
    stations = "Wear",                      # Palatina stations
    variables = c("ta","slp"),              # "temp", "slp"
    distance = "RMSE",                      # "Euclid","RMSE","Mahalanobis", RMSE and weighted Euclid are the same!
    WT_sel=.95,                             # preselection of weather types (number or probability of WT)
    nAnalog = 50,                           # number of analogs
    startRef = 1950,                        # start of reference period
    endRef = 2020,                          # end of reference period
    startCal = "1788-11-01",                # calibration start year
    endCal = "2020-12-31",                  # calibration end year
    dayselection = 30,                      # analog dates only +-dayselection days from target date  
    standardize = T,                        # standardize data?
    trend = T,                              # correct temperature trend (F,T="3rd order polynomial")
                                            # for period 1950-2018
    seasonality = T,                        # correct temperature seasonality
    window=5                                # number of days around cal period not to consider in analog pool
)              

### load data 
# (one file with all data used (dates, weather types with probability and measurements))
# make sure that dates column is in a date format, rest numeric or integer
load("data_input/analogue_data.RData")

### load packages
source("01_packages.R")


###################################################################################################
### ANALOGUE RESAMPLING METHOD
###################################################################################################
### execute preprocessing according to specifications
source("02_preprocessing.R")

## calculate analogues
source("03_analogs.R")


###################################################################################################
### KALMAN FITTING
###################################################################################################
# define setting for Kalman fitting
kalmanset <- list(
  analog_dates = "output/ARM_run_2019-06-21/2019-06-21_analog.dates_1864-01-01-2017-12-31_deseas_RMSE.txt", # define analog dates
  station_data = "data_input/analogue_data.RData",           # define station data (TOT)
  station_xy = "data_input/NBCN_clima.csv", # define station coordinate file
  input_raster = "data_input/TabsD_1961-2017_2018-10-16.nc", # define input raster (tabs)
  input_raster_deseas = "data_input/TabsD_deseas_1961-2017_2018-10-19.nc",
  climatology = "data_input/TabsD_climatology.grd",
  stationset = c("BAS","BER","CDF","DAV","ENG","GSB","GVE","LUG","LUZ","MER","NEU","OTL","RAG","SAE","SAM",
                 "SBE","SIA","SMA","STG"), # define set of stations to do kalman filter
  # stationset = c("all"), # define set of stations to do kalman filter
  localize = F,
  cutoff = 200, # define horizontal cutoff distance L [km] (if localize = T)
  Z_cutoff =2000, # define vertical cutoff distance z [m]  (if localize = T)
  R = 1 # define measurement error R [°K]
)

# define year for which EnKF is run (for multiple years use bash script EnKF_parallel.sh from console)
in_year=1961

### run EnKF-Script
source("05_EnKF.R")


###################################################################################################
### QUANTILE MAPPING
###################################################################################################
qmapset <- list(
  input_raster_obs = "data_input/RhiresD_1961-2017_2018-10-16.nc",
  input_raster_rec = "output/ARM_run_2019-06-21/2019-06-21_precip_analogs_1864-01-01-2017-12-31_RMSE.nc",
  method="PTF",
  transfun="expasympt",
  cost="RSS",
  monthly=F,
  wett.day=0.1
)

source("06_Qmap.R")


### END