library(dataresqc)
library(ncdf4)
library(zoo)

###################
### READ/FORMAT ZONAL MEANS 1950-2020 & 1751-1800 AND CALCULATION OF CLIMATE CHANGE 'OFFSET'
###################

# set path and filename
zonmean17511800 <- "~/paper2/Data/Preprocessed/zonmean_ta_17511800.nc"
zonmean19502003 <- "~/paper2/Data/Preprocessed/zonmean_ta_19502003.nc"
dname <- "air_temperature"
zonpath <- list(zonmean17511800,zonmean19502003)

for (i in 1:length(zonpath)) {
  # open a netCDF file
  ncin <- nc_open(zonpath[[i]])
  print(ncin)

  # get longitude and latitude
  lon <- ncvar_get(ncin,"lon")
  nlon <- dim(lon)
  head(lon)
  lat <- ncvar_get(ncin,"lat")
  nlat <- dim(lat)
  head(lat)

  # get temperature
  tmp_array <- ncvar_get(ncin,dname)
  dlname <- ncatt_get(ncin,dname,"long_name")
  dunits <- ncatt_get(ncin,dname,"units")
  fillvalue <- ncatt_get(ncin,dname,"_FillValue")
  dim(tmp_array)

  # replace netCDF fill values with NA's
  tmp_array[tmp_array==fillvalue$value] <- NA
  length(na.omit(as.vector(tmp_array)))
  
  if (i == 1) {
    zonmean18th <- round(t(tmp_array), 4)
    colnames(zonmean18th) <- round(lat, 4)
    mean18th <- colMeans(zonmean18th)
  } else {
    zonmean20th <- round(t(tmp_array), 4)
    colnames(zonmean20th) <- round(lat, 4)
    mean20th <- colMeans(zonmean20th)
  }
}
#View(mean18th)
#View(mean20th)

# calculate offset
offset <- mean20th - mean18th; offset <- t(data.frame(offset))
#View(offset)


###################
### SUBTRACT OFFSET FROM REFERENCE SERIES and WRITE NEW FILE
###################
refpath <- "~/paper2/Data/Preprocessed/2_Detrended/ref/"
ref.files <- list.files(refpath, full.names = T)

for (k in 1:length(ref.files)) {
  ref <- read_sef(ref.files[k], all=T)
  meta <- data.frame(t(read_meta(ref.files[k])))
  
  # find closest ekf latitude to station latitude
  ind <- as.numeric(which.min(abs(as.numeric(meta$lat) - as.numeric(colnames(offset)))))
  
  # apply offset
  ref$offset <- round(ref$Value - offset[,ind], 2)
  
  newref.path <- "~/paper2/Data/Preprocessed/3_Offset/ref/"
  
  # write new SEF files
  write_sef(Data = ref[,c(2:6,10)],
            outpath = newref.path,
            variable = meta["var"],
            cod = meta["id"],
            nam = meta["name"],
            lat = meta["lat"],
            lon = meta["lon"],
            alt = meta["alt"],
            link = meta["link"],
            sou = meta["source"],
            units = meta["units"],
            stat = meta["stat"],
            metaHead = meta["meta"],
            meta = paste0(ref$Meta,"|offset=",round((-1)*offset[,ind], 2)),
            period = "day",
            time_offset = 0,
            keep_na = TRUE,
            note="detrend_offset")
}


###################
### CREATE NCDF FILE WITH 'OFFSET' VECTOR THAT WILL BE SUBTRACTED FROM GRIDDED DATA IN THE TERMINAL
###################

# offset vector also calculated in terminal with CDOs, used that to deal with gridded dataset