## Plot correlation or variance vs distance for a monthly dataset in SEF

library(dataresqc)
library(geosphere)
library(lubridate)

anomalies <- TRUE # use anomalies or absolute values
variable <- "p" # change depending on
minlength <- 120 # minimum overlapping period in months
maxyear <- 2020

if (variable == "p") {
  files <- list.files("../../Palatina/p",full.names=TRUE)
} else if (variable == "mslp") {
  files <- list.files("../../Palatina/mslp",full.names=TRUE)
} else if (variable == "ta") {
  files <- list.files("../../Palatina/ta",full.names=TRUE)
#  files <- files[seq(1,length(files),10)]
#  inventory <- read.csv("/scratch3/PALAEO-RA/Instrumental/SEF/Monthly_means/TA_inventory.tsv",
#                        sep="\t", quote="", comment.char="", stringsAsFactors=FALSE)
#  inventory <- inventory[order(inventory$end,na.last=TRUE), ]
#  files <- inventory$filepath[sample(1:nrow(inventory),1000)]
#  files <- inventory$filepath
}
n <- length(files)


## Function to calculate anomalies
anom <- function(x, perc=FALSE) {
  xclim <- aggregate(x[,3], list(x[,1]), mean, na.rm=TRUE)
  for (i in 1:12) {
    if (perc) {
      x[x[,1]==i,3] <- 100 * (x[x[,1]==i,3] - xclim$x[xclim[,1]==i]) / xclim$x[xclim[,1]==i]
    } else {
      x[x[,1]==i,3] <- x[x[,1]==i,3] - xclim$x[xclim[,1]==i]
    }
  }
  return(x)
}


## Read all data and metadata into a list
Data  <- list()
for (i in 1:n) {
  Data[[i]] <- list()
  Data[[i]]$coords <- as.numeric(read_meta(files[i],c("lon","lat")))
#  if (grepl("meteo.ru",files[i])) Data[[i]]$coords <- rev(Data[[i]]$coords)
  x <- read_sef(files[i], all=TRUE)
  x <- x[!grepl("qc",x$Meta), ]
  x <- x[which(x$Year<=maxyear), ]
  Data[[i]]$dates <- ISOdate(x$Year, x$Month, 1)
  if (anomalies & nrow(x)>=minlength) {
    x <- x[, c("Month","Year","Value")]
    Data[[i]]$ta <- anom(x)[,3]
  } else Data[[i]]$ta <- x$Value
}
if (read_meta(files[i],"stat") == "mid_range" | grepl("Calendar",read_meta(files[i],"meta"))) {
  Data[[i]]$ta[1:length(x$Value)] <- NA
}


## Calculate distance and correlation for every pair of stations
cormat <- distmat <- array(dim=c(n,n))
fitvec <- meanyear <- array(dim=n)
for (i in 1:n) {
  if (i %in% seq(100,20000,100)) message(i)
  for (j in min((i+1),n):n) {
  #for (j in 1:n) {
    if (i != j) {
      #distmat[i,j] <- distGeo(Data[[i]]$coords, Data[[j]]$coords) / 1000
      distmat[i,j] <- (distGeo(Data[[i]]$coords, Data[[j]]$coords) / 1000)**2
      if (is.na(distmat[i,j])) distmat[i,j] <- 1e+12
      x <- Data[[i]]$ta[which(Data[[i]]$dates %in% Data[[j]]$dates)]
      y <- Data[[j]]$ta[which(Data[[j]]$dates %in% Data[[i]]$dates)]
      if (sum(!is.na(x-y)) >= minlength & distmat[i,j]<1000**2) {
        if (sd(x,na.rm=TRUE) > 0 & sd(y,na.rm=TRUE) > 0) {
          #cormat[i,j] <- cor(x, y, "pairwise.complete.obs", method="spearman")
          cormat[i,j] <- var(x-y, use="pairwise.complete.obs")
        }
      }
    }
  }
  # Linear fit (for error estimation after Wartenburger et al.)
  y <- cormat[i,]
  y[which(y>ifelse(variable=="p",50,10))] <- NA
  if (sum(!is.na(y)) >= 10) {
    fitvec[i] <- lm(y ~ distmat[i,])$coefficients[1] / 2
    meanyear[i] <- year(mean(Data[[i]]$dates[which(!is.na(Data[[i]]$ta))]))
  }
}


## Plot
#plot(c(distmat), c(cormat), xlim=c(0,12000), ylim=c(-0.5,1), xlab="Distance [km]", ylab="Correlation")
plot(c(distmat[145,]), c(cormat[145,]), xlim=c(0,2000000), ylim=c(0,50), xlab="Distance² [km²]", ylab="Variance [hPa²]")
print(quantile(fitvec, c(0.1,0.5,0.9), na.rm=TRUE))
hist(fitvec)
plot(meanyear, fitvec, ylab="sigma²", main=variable, panel.first=grid())
fit <- lm(fitvec ~ meanyear)$coefficients
#years <- min(meanyear,na.rm=T):max(meanyear,na.rm=T)
#lines(years, fit[1] + fit[2]*years, col="red")
#out <- trunc(which(c(distmat)>2000 & c(cormat)>0.9)/ncol(distmat))+1
#out <- inventory[out,c("filepath","latitude","longitude","station")]
#out <- out[!duplicated(out$filepath),]
#write.table(out, "suspicious_coords.tsv", sep="\t", row.names=FALSE, quote=FALSE)
